% First the one step approach is used for computing a trajectory. 
% Then, this trajectory is compared with the one iteratively computed in
% which just three waypoints are fed at a time to the path planner.
% Notice that the iterative approach differs with the real data because the
% trajectory is recomputed at different time.

clear;close all;clc;

% Parameters

N       =   5;% order of the polynomial
Wpx     =   [-.47 -.53 0.71 1.1 3.61 4.12 6.21 6.69 7.24 7.27 6.2 5.83 3.19 2.66 0.53 0.11];
Wpy     =   [-1.25 -1.84 -3.04 -3.46 -3.67 -3.66 -3.18 -2.81 -0.91 -.31 1.2 1.72 1.81 1.68 1.22 0.79];
Wpz     =   0.90+0.3*sin(2*pi/0.25*Wpx);%0.90*ones(1,17);
WP      =   [[0, Wpx]',[0,Wpy]',[0,Wpz]']; %Waypoints
v_max   =   0.35;%m/s vo 0.25
a_max   =   2.5;%m/s^2
vi      =   zeros(1,3); %initial velocity
alfa    =   0.25; %modul of the final velocity vector

% Width of each corridor
Wid     =   size(2,size(WP,1));
for k = 1:size(WP,1)-1
    Wid(1,k) = k;
    Wid(2,k) = 0.35;
end
Delta = 0.05;
%%
%One step approach: all the waypoints are sent together to the path planner module
tic

[ottim_flag, T, q, q_vel, q_acc, yaw, time]=trajectory_planning(WP,...,
    N,Delta,vi,alfa,v_max,a_max,Wid);

toc

figure(1);clf;
subplot(2,1,1)
plot(q(:,1),q(:,2))
hold on
plot(WP(:,1),WP(:,2),'rx')
axis equal
xlabel('x(t) [m]')
ylabel('z(t) [m]')
grid on
subtitle('Top view','FontSize',14)
subplot(2,1,2)
plot3(q(:,1),q(:,2),q(:,3))
hold on
plot3(WP(:,1),WP(:,2),WP(:,3),'rx')
xlabel('x(t) [m]');ylabel('z(t) [m]');zlabel('y(t) [m]');
grid on
subtitle('3D view','FontSize',14)

figure(2);clf;
subplot(2,1,1)
plot(time,q_vel(:,1),'b')
hold on
plot(time,q_vel(:,2),'r')
hold on
plot(time,q_vel(:,3),'g')
plot(time,v_max*ones(1,length(time)),'k--')
hold on
plot(time,-v_max*ones(1,length(time)),'k--')
xlabel('Time (s)')
ylabel('Velocity (m/s)')
legend('dx','dz','dy')
grid on
subtitle('Physical bound on velocity')
subplot(2,1,2)
plot(time,q_acc(:,1),'b')
hold on
plot(time,q_acc(:,2),'r')
hold on
plot(time,q_acc(:,3),'g')
plot(time,a_max*ones(1,length(time)),'k--')
hold on
plot(time,-a_max*ones(1,length(time)),'k--')
legend('ddx','ddz','ddy')
xlabel('Time (s)')
ylabel('Acceleration (m/s^2)')
grid on
subtitle('Physical bound on acceleration')

figure(3);clf;
plot(time,yaw*180/pi)
grid on
xlabel('Time (s)')
ylabel('yaw (deg)')

figure(4);clf;
subplot(3,1,1)
plot(time,q(:,1))
hold on
plot([0 cumsum(T)], WP(:,1),'rx')
xlabel('Time [s]');ylabel('x(t) [m]')
%
subplot(3,1,2)
plot(time,q(:,2))
hold on
plot([0 cumsum(T)], WP(:,2),'rx')
xlabel('Time [s]'); ylabel('z(t) [m]')
%
subplot(3,1,3)
plot(time,q(:,3))
hold on
plot([0 cumsum(T)], WP(:,3),'rx')
xlabel('Time [s]'); ylabel('y(t) [m]')

%% iterative approach

WP  =  [Wpx',Wpy',Wpz'];

figure(5);clf;
plot3(q(:,1),q(:,2),q(:,3),'b--')
hold on
for k=1:(size(WP,1)-1)/2+1
    if k==1
        WP_subset=[0 0 0;WP(2*k-1,:);WP(2*k,:)];
        vi=zeros(1,3);
        [~, ~, coord,vel]=trajectory_planning(WP_subset,N,Delta,vi,alfa,v_max,a_max,Wid);
        plot3(coord(:,1),coord(:,2),coord(:,3),'k')
        hold on
    else
        vi=vel(end,:);
        initial_state=[coord(end,1),coord(end,2),coord(end,3)];
        WP_subset=[initial_state;WP(2*k-1,:);WP(2*k,:)];
        [~, ~, coord,vel]=trajectory_planning(WP_subset,N,Delta,vi,alfa,v_max,a_max,Wid);
        plot3(coord(:,1),coord(:,2),coord(:,3),'k')
        hold on
    end
end
hold on
plot3([0 ;WP(:,1)],[0 ;WP(:,2)],[0 ;WP(:,3)],'rx')
grid on
xlabel('x(t) [m]');ylabel('z(t) [m]');zlabel('y(t) [m]');
legend('one step','iterative') 